function net = chmm_construct(NM, NH, NO, discrete, data)

% net = chmm_construct(NM, NH, NO, discrete)
%    NM - number of models
%    NH - number of hidden states
%    NO - number of observation symbols
%    discrete - 1 if observations are discrete 0 if continuous
%    data - optional, better initialization can be obtained when
%           data is provided
%    net - HMM network to be constructed

if nargin < 4, discrete = 1; end

% General parameters
assert(NM >= 2, 'at least 2 models for coupled models');
if length(NH) > 1
   nhid = NH;
else
   nhid = NH(:,ones(1,NM)); % number of hidden states [NH1 NH2 NH3]
end
if length(NO) > 1
   nout = NO;
else
   nout = NO(:,ones(1,NM)); % number of observation symbols [NO1 NO2 NO3]
end

ss = NM * 2; % slice size
dnodes = 1:ss; % discrete nodes
onodes = NM+1:ss; % observation nodes

intra = zeros(ss);
for i=1:NM
   intra(i,i+NM)=1;
end
inter = zeros(ss);
for i=1:NM
   inter(i,1:NM)=1; % fully coupled
end

node_sizes = [nhid nout];
eclass1 = 1:ss; % time slice 1
eclass2 = [ss+1:ss+NM NM+1:ss]; % time slice 2 and up
%eclass = [eclass1 eclass2];

% make the chmm model structure
net = mk_bn(intra, inter, node_sizes, dnodes, eclass1, eclass2, discrete);

if discrete==0 & nargin==5
   
   [pi, aij, cjk, mu, sigma] = init_mhmm(data,NH,NO,'diag',0);
   pi = repmat(pi,[1,NM]);
   aij = repmat(aij,[1,1,NM,NM]);
   bjo = zeros(NH,NO,3,NM);
   bjo(:,:,1,:) = repmat(reshape(cjk,[NH,NO,1,1]),[1,1,1,NM]);
   mu = reshape(mu,[NM,NH*NO])';
   bjo(:,:,2,:) = reshape(mu,[NH,NO,1,NM]);
   for i = 1 : NM
      bjo(:,:,3,i) = reshape(sigma(i,i,:,:),[NH,NO,1,1]);
   end
   
   net = update_bnet(net,pi,aij,bjo,net.theta);
end

return;

