function [alpha, scale, ll] = chmm_forward(net, obs, scale_in)

% [alpha, scale] = chmm_forward(net, obs, scale_in)
%    net - chmm model
%    obs - observation sample(s)
%    scale_in - input scaling coefficients (optional)
%    alpha - forward variable
%    scale - associated scaling coefficients
%    ll - log-likelihood

ss = net.slice_size;
nh = ss/2;
ns = net.node_sizes(:);
[no,T] = size(obs);
assert(no==nh, '# of samples and # of models do not match');

alpha = cell(1,nh);
for i = 1 : nh
   alpha{i} = zeros(ns(i),T);
end
if nargin == 2
   scale = ones(1,T);
else
   scale = scale_in;
end

%%%%% forward algorithm --> begin %%%%%
for j = 1 : nh %% initialization
	ocpt = net.CPD{j+nh}.CPT; % output node obs matrix
   if net.discrete == 1
      alpha{j}(:,1) = net.CPD{j}.prior' .* ocpt(:,obs(j,1));
   else
      bjot = cobs_prob(ocpt,obs(j,1));
      bjt = sum(bjot,2);
      alpha{j}(:,1) = net.CPD{j}.prior' .* bjt;
   end
end
if nargin==2
   scale(1) = 1/sum(alpha{1}(:,1));
   for j = 1 : nh
      alpha{j}(:,1) = alpha{j}(:,1) * scale(1);
   end
end

for t = 2 : T %% induction
   %alpha_bak = alpha;
   for j = 1 : nh
      %%% transition matrices
      cpd = struct(net.CPD{net.equiv_class(j,2)});
      %%% find out parents
      %p = cpd.parents;
   	tmp = zeros(ns(j),nh);
      for ip = 1:nh
         tmp(:,ip) = cpd.CPT{ip}' * alpha{ip}(:,t-1);
      end
      %%% output node obs matrix 
      ocpt = net.CPD{j+nh}.CPT;
      if net.discrete == 1
         x = ocpt(:,obs(j,t));
      else
         bjot = cobs_prob(ocpt,obs(j,t));
         x = sum(bjot,2);
      end
      alpha{j}(:,t) = tmp * cpd.theta';
	   alpha{j}(:,t) = alpha{j}(:,t) .* x;
   end
   if nargin==2
      scale(t) = 1/sum(alpha{1}(:,t));
      for j = 1 : nh
         alpha{j}(:,t) = alpha{j}(:,t) * scale(t);
      end
   end
end
%%%%% forward algorithm <-- end %%%%%

ll = 0;
for j = 2 : nh
   ll = ll + log(sum(alpha{j}(:,T)));
end
ll = ll - sum(log(scale))*nh;

return;
