function [alpha, scale, ll] = chmm_forward2(net, obss, scale_in)



% [alpha, scale, ll] = chmm_forward2(net, obs, scale_in)

%    net - chmm model

%    obs - observation sample(s)

%    scale_in - input scaling coefficients (optional)

%    alpha - forward variable

%    scale - associated scaling coefficients

%    ll - log-likelihood



ss = net.slice_size;

nh = ss/2;

ns = net.node_sizes(:);

[nobs,T,nsamp] = size(obss);

assert(nobs==nh, '# of samples and # of models do not match');



if nargin == 2

   scale = ones(1,T);

else

   scale = scale_in;

end



ns1 = ns(1);

no = ns(1+nh);

ns2 = ns1 * ns1;

nso = ns1 * no;



n1 = cumprod([1 ns(1:nh)']); 

np = n1(nh+1); 

n2 = np ./ n1(2:nh+1); 

pi = zeros(np,nh);

for i = 1 : nh 

   a = repmat(1:ns(i), n1(i), 1); 

   b = a(:); 

   pi(:,i) = repmat(b,n2(i),1); 

end

%keyboard;



aa = zeros(nh);

cc = (1:nh)';

hh = (1:ns1);

ip = pi';

pp = zeros(np,np,nh);

pb = zeros(np,nh);

B = zeros(np);



% calculation of sum(theta * aij)

ia1 = repmat(pi,[np,1]);

ia3 = (cc'-1)*ns2;

for c = 1 : nh

   thc = net.theta(:,c);

   ia4 = ia3 + (c-1)*ns2*nh;

   ia2 = repmat((ip(c,:)-1)*ns1,[np,1,nh]);

   ia2 = reshape(ia2,[np*np,nh]);

   ia = repmat(ia4,[np*np,1]);

   ia = ia + ia1 + ia2;

   ia2 = net.aij(ia);

   pp(:,:,c) = reshape(ia2*thc, [np,np]);

end

B2 = prod(pp,3)';



clear pp hh ia ia1 ia2 ia3 ia4;



ibjk1 = ip + repmat((cc-1)*nso,[1,np]);

ibjk2 = ip + repmat((cc-1)*ns1,[1,np]);



alpha = zeros(np,nsamp);

bjot = zeros(ns1,no,nh);

bjt = zeros(ns1,nh);

for n = 1 : nsamp

   obs = obss(:,:,n);

   %%%%% extended forward algorithm --> begin %%%%%

   % initialization

   if net.discrete == 1

      ib = (obs(:,1)-1)*ns1; % + (cc-1)*nso;

      ic = repmat(ib,[1,np]);

      ic = ic + ibjk1;

      pb1 = net.bjk(ic); %%% bjk's

   else

      for c = 1 : nh

         bjot(:,:,c) = cobs_prob(net.bjo(:,:,:,c),obs(c,1));

	 bjt(:,c) = sum(bjot(:,:,c),2);

      end

      pb1 = bjt(ibjk2);

   end



   ic = repmat((cc-1)*ns1,[1,np]);

   ic = ic + pi';

   pp1 = net.pi(ic); %%% pi's



   alpha(:,n) = prod(pb1)';

   alpha(:,n) = alpha(:,n) .* prod(pp1)';

   

   if nargin==2 & n==1, scale(1) = 1/sum(alpha(:,n)); end

   alpha(:,n) = alpha(:,n) * scale(1);

   

   % induction

   for t = 2 : T

      if net.discrete == 1

         oc = obs(:,t);

         for c = 1 : nh

            a = net.bjk(:,oc(c),c)';

            b = repmat(a,[n1(c),1]);

            pb(:,c) = repmat(b(:),[n2(c),1]);

         end

      else

         for c = 1 : nh

            bjot(:,:,c) = cobs_prob(net.bjo(:,:,:,c),obs(c,t));

	    bjt(:,c) = sum(bjot(:,:,c),2);

	    a = bjt(:,c)';

	    b = repmat(a,[n1(c),1]);

	    pb(:,c) = repmat(b(:),[n2(c),1]);

         end

      end

      B1 = prod(pb,2);

      alpha(:,n) = B2 * alpha(:,n);

      alpha(:,n) = alpha(:,n) .* B1;

      %% scaling

      if nargin==2 & n==1, scale(t) = 1/sum(alpha(:,n)); end

      alpha(:,n) = alpha(:,n) * scale(t);

   end

   %%%%% extended forward algorithm <-- end %%%%%

end



ll = sum(log(sum(alpha,1)))/nsamp - sum(log(scale));



return;

