function [net, ll, err] = chmm_train_exact(obss, NM, NH, NO, nIter, discrete, net_in, coupled)

% [net,ll,err] = chmm_train_exact(obs,NM,NH,NO,nIter,discrete,net_in,coupled);
% Train a CHMM model based on exact Baum-Welch algorithm based on the
% true likelihood function. Number of forward/backward variables is
% exponential in the number of HMM models coupled together.
% INPUT:
%    obss - observation samples (multi-dimensional array, NH by T by nobs)
%    NM - number of models coupled
%    NH - number of hidden states
%    NO - number of observation symbols
%    nIter - number of iterations
%    discrete - is the observation discrete?
%    net_in - optional starting net
%    coupled - specifying whether or not the HMMs are coupled
% OUTPUT:
%    net - trained HMM network
%    ll - log-likelihood at each iteration
%    err - err is 1 if singularity problem occurs, 0 otherwise

err = 0;
if nargin < 2, NM = 2; end
if nargin < 3, NH = 3; end
if nargin < 4, NO = 5; end
if nargin < 5, nIter = 20; end
if nargin < 6, discrete = 1; end
if nargin < 7
   net = chmm_construct(NM,NH,NO,discrete);
   if discrete == 1
      save net0 net; % save starting network
   else
      save net0c net;
   end
else
   net = net_in;
end
if nargin < 8, coupled = 1; end

if coupled == 0
   net = decouple_all(net);
end

% initialize
[nh,T,nobs] = size(obss);
assert(nh==NM);

%nIter = 20;
ll = zeros(1,nIter);
%ll = zeros(2,nIter);
% iteration
for i = 1 : nIter % begin of outer loop

   [pi,aij,bjk,th,alpha,sa,beta,sb,err] = chmm_fbp(net,obss);
   if err, break; end
   ll(1,i) = sum(log(reshape(sum(alpha(:,T,:),1),[1,nobs])))/nobs - sum(log(sa));
   %disp(sprintf('i = %d, log likelihood is %10.4f', i, ll(1,i)));
   %if ~isempty(find(bjk(:,:,3,:)<=0)), keyboard; end
   
   % replacement
   net = update_bnet(net,pi,aij,bjk,th);
   %net = update_bnet(net,pi,aij,net.bjo,th);
   
end; % end of outer loop

return;

