function cpd = get_CPD(net, i)
%
% cpd = get_CPD(net, i)
%
%    get the Conditional Probability Distribution of node i in model net
%    assigned
%

ns = net.node_sizes(:);
ss = net.slice_size;
nh = ss/2;

cpd.self = i;
cpd.parents = [];
cpd.CPT = [];
cpd.prior = [];
cpd.theta = [];

if i <= nh % hidden node in slice 1
   cpd.prior = stochastic(rand(1,ns(i)));
elseif i <= ss % output node
   cpd.parents = find(net.intra(:,i)==1);
   assert(length(cpd.parents)==1, 'incorrect number of parents for obs node');
   if net.discrete==1
      cpd.CPT = stochastic(rand(ns(cpd.parents),ns(i)));
      cpd.CPT = difobs(cpd.CPT);
   else
      cpd.CPT = rand(ns(cpd.parents),ns(i),3);
      cpd.CPT(:,:,1) = stochastic(cpd.CPT(:,:,1),2);
      cpd.CPT(:,:,1) = difobs(cpd.CPT(:,:,1));
      cpd.CPT(:,:,2) = cpd.CPT(:,:,2)*5;
      cpd.CPT(:,:,3) = cpd.CPT(:,:,3)*2+1;
   end
else % hidden state in slice 2 and after
   cpd.parents = find(net.inter(:,i-ss)==1)';
   np = length(cpd.parents);
   assert(np>=1, 'incorrect number of parents for hidden node');
   if np == 1 % single or non-coupled
      cpd.CPT = stochastic(rand(ns(cpd.parents),ns(i)));
      cpd.CPT = difobs2(cpd.CPT);
   else % coupled
      cpd.CPT = cell(1,np);
      for k = 1 : np
         cpd.CPT{k} = stochastic(rand(ns(cpd.parents(k)),ns(i)));
         cpd.CPT{k} = difobs2(cpd.CPT{k});
      end
      % set coupling matrix to identity matrix
      cpd.theta = stochastic(rand(1,np));
      j = find(cpd.parents==i-ss);
      cpd.theta(j) = cpd.theta(j) + 1;
      cpd.theta = cpd.theta / 2;
   end
end

return;
