function bnet = mk_bn(intra, inter, node_sizes, dnodes, eclass1, eclass2, discrete)

% bnet = mk_bn(intra, inter, node_sizes, dnodes, ecalss1, eclass2)
%
%    bnet       - the output bayesian net structure, in which the prob matrices
%                 have not been initialized yet
%    intra      - the connections within one time slice
%    inter      - inter-connections between two slices
%    node_sizes - the sizes of all nodes
%    dnodes     - discrete nodes
%    eclass1    - node class number in slice 1
%    eclass2    - node class number in slice 2 and after
%    discrete   - 1 if the observations are discrete, 0 other wise

assert(size(intra,1)==size(intra,2), 'Incorrect intra parameter');
n = length(intra); % number of nodes in one time slice
nh = n/2; % number of hidden nodes (i.e. models)

if nargin < 4, dnodes = 1:n; end % discrete nodes
if nargin < 5, eclass1 = 1:n; end % 1st slice
if nargin < 6, eclass2 = [(n+1):(n+nh) (nh+1):n]; end % 2nd slice and after
if nargin < 7, discrete = 1; end

% construct a struct bnet
bnet.slice_size = n;
bnet.intra = intra;
bnet.inter = inter;
bnet.discrete = discrete;
ns = node_sizes(:);
ns1 = ns(1);
no = ns(nh+1);
assert(length(ns)==n,'mismatched number of nodes');
bnet.node_sizes = [ns ns];

cnodes = setdiff(1:n, dnodes);
bnet.dnodes = dnodes;
bnet.cnodes = cnodes;

bnet.equiv_class = [eclass1(:) eclass2(:)];
nc = max(bnet.equiv_class(:));

bnet.pi = zeros(ns1,nh);
bnet.aij = zeros(ns1,ns1,nh,nh);
bnet.bjk = zeros(ns1,no,nh);
bnet.bjo = zeros(ns1,no,3,nh); % continuous parameters, c_jk, u_jk, U_jk
bnet.theta = zeros(nh);

bnet.CPD = cell(1,nc);
for i = 1 : nc
   %bnet.CPD{i} = get_CPD1(bnet,i);
   bnet.CPD{i} = get_CPD(bnet,i);
   if i <= nh, bnet.pi(:,i) = (bnet.CPD{i}.prior)'; end
   if i > nh & i <= n
      if discrete==1
         bnet.bjk(:,:,i-nh) = bnet.CPD{i}.CPT;
      else
         bnet.bjo(:,:,:,i-nh) = bnet.CPD{i}.CPT;
      end
   end
   if i > n
      if nh > 1
         bnet.theta(:,i-n) = (bnet.CPD{i}.theta)';
         for j = 1:nh
            bnet.aij(:,:,j,i-n) = bnet.CPD{i}.CPT{j};
         end
      else
         bnet.aij(:,:,1,1) = bnet.CPD{i}.CPT;
      end
   end
end

return;

